/*	$LAAS$
 *
 * Copyright (c) 2000,2008 LAAS/CNRS                       --  Fri Aug  4 2006
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
#ifndef H_CALIFE_STEREO_PIXEL
#define H_CALIFE_STEREO_PIXEL

#include "libimages3d.h"

#ifdef __cplusplus
extern "C" {
#endif

/* Indique le type de rduction d'image */
typedef enum {SP_REDUCTION_SOUS_ECHANTILLONAGE,
	      SP_REDUCTION_MOYENNAGE} SP_TYPE_REDUCTION;

/* Indique si l'on fait ou non la phase de rectification */
typedef enum {SP_RECTIFICATION_NIL,
	      SP_RECTIFICATION_MATRICES,
	      SP_RECTIFICATION_TABLES,
	      SP_RECTIFICATION_INVERSE} SP_TYPE_RECTIFICATION;

/* Indique si l'on corrige la distortion o non pendant la phase de
   rectification, et le type de correction. */
typedef enum {SP_DISTORTION_NIL,
	      SP_DISTORTION_R_OLD,
	      SP_DISTORTION_R,
	      SP_DISTORTION_R3_T2} SP_CORRECTION_DISTORTION;

/* Indique si l'on interpole o non pendant la phase de rectification */
typedef enum {SP_INTERPOLATION_NIL,
	      SP_INTERPOLATION} SP_TYPE_INTERPOLATION;

/* Indique le type de transformation census qu'on applique */
typedef enum {SP_CENSUS_NIL,
	      SP_CENSUS_8BITS_SQUARE,
	      SP_CENSUS_8BITS_DIAMOND,
	      SP_CENSUS_MEAN_8BITS_SQUARE,
	      SP_CENSUS_MEAN_8BITS_DIAMOND,
	      SP_CENSUS_MEAN_MEAN_8BITS_SQUARE,
	      SP_CENSUS_MEAN_MEAN_8BITS_DIAMOND} SP_TYPE_CENSUS;

/* Indique quelle image afficher dans la structure 'images_tmp' ou la
   structure 'images_results' */
typedef enum {SP_DISPLAY_INTERMEDIAIRE_HAUT,
	      SP_DISPLAY_INTERMEDIAIRE_BAS,
	      SP_DISPLAY_INTERMEDIAIRE_MOYENNES_1,
	      SP_DISPLAY_INTERMEDIAIRE_MOYENNES_2,
	      SP_DISPLAY_MOYENNES,
	      SP_DISPLAY_SIGMAS,
	      SP_DISPLAY_DISPARITES,
	      SP_DISPLAY_SIGMA_DISPARITES,
	      SP_DISPLAY_REGIONS} SP_TYPE_DISPLAY;

/* Indique le type de corrlation que l'on fait */
typedef enum {SP_CORRELATION_ZNCC,
	      SP_CORRELATION_ZNCC_SIGMA,
	      SP_CORRELATION_CENSUS,
	      SP_CORRELATION_CENSUS_SIGMA} SP_TYPE_CORRELATION;

/* Indique le type d'interpolation pour trouver la disparite sub-pixellique */
typedef enum {SP_INTERPOLATION_DISP_NIL,
	      SP_INTERPOLATION_DISP_PARABOLIQUE_ONLY,
	      SP_INTERPOLATION_DISP_BILINERAIRE_ONLY,
	      SP_INTERPOLATION_DISP_BOTH} SP_TYPE_INTERPOLATION_DISP;

/* Indique le repre dans lequel on fait la reconstruction */
typedef enum {SP_REPERE_RECTIFIE,
	      SP_REPERE_CAMERA} SP_REPERE_RECONSTRUCTION;

/* Indique si on dtermine les variances pendant la reconstruction */
typedef enum {SP_RECONSTRUCTION_NO_SIGMA,
	      SP_RECONSTRUCTION_SIGMA} SP_TYPE_RECONSTRUCTION;

/* Pour l'tat d'une structure 'SP_IMAGES_SIZES' */
typedef enum SP_IMAGES_SIZES_STATE {SP_IMAGES_SIZES_NOT_INITIALIZED,
				    SP_IMAGES_SIZES_INITIALIZED} SP_IMAGES_SIZES_STATE;

/* Structure  dans laquelle sont stockes toutes les variables qui spcifient
   la taille des images  traiter. Seuls les champs 'taux_reduction', 'debut_'
   et 'fin_' sont modifiables par l'utilisateur : tous les autres en sont
   dduits. */
typedef struct SP_IMAGES_SIZES {
  SP_IMAGES_SIZES_STATE	state;	/* tat de la structure */
  int		nb_lig_original;
  int		nb_col_original;
  int		taille_originale; /* Taille des images originales */

  int		taux_reduction_i;
  int		taux_reduction_j; /* Taux de rduction des images suivant les
				     lignes et les colonnes */

  int           previousReduction; /* Reduction that have been applied to the
				      images before their processing (typically,
				      at the camera hardware level) */

  double	debut_ligne;
  double	fin_ligne;
  double	debut_colonne;
  double	fin_colonne;	/* Dbut et fin de la sous-image (en
				   pourcentage, valeurs de 0  1)*/

  int		nb_lig_reduit;
  int		nb_col_reduit;
  int		taille_reduite; /* Taille des images aprs rduction (sans
				   crop) */

  int		nb_lig_crop;
  int		nb_col_crop;
  int		taille_crop;	/* Taille des images aprs crop (sans
				   rduction) */

  int		i_min_orig;
  int		i_max_orig;
  int		j_min_orig;
  int		j_max_orig;	/* Coordonnes des sous images DANS L'IMAGE
				   ORIGINALE */
  int		i_min;
  int		i_max;
  int		j_min;
  int		j_max;		/* Coordonnes des sous images DANS L'IMAGE
				   REDUITE */
  int		nb_lig_final;
  int		nb_col_final;

  int		taille_finale; /* Taille 'finale' des images, aprs
				   reduction et crop */
} SP_IMAGES_SIZES;



/**********************************************************************/
/* Pour l'tat d'une structure 'SP_IMAGES_TMP' */
typedef enum SP_IMAGES_TMP_STATE {
  SP_IMAGES_TMP_NOT_ALLOCATED,
  SP_IMAGES_TMP_ALLOCATED} SP_IMAGES_TMP_STATE;

/* Pour l'tat des diffrentes images de la structure 'SP_IMAGES_TMP' */
typedef enum SP_IMAGE_STATE {
  SP_IMAGE_ALLOCATED,
  SP_IMAGE_INITIALIZED,
  SP_IMAGE_COMPUTED} SP_IMAGE_STATE;


/* Structure  dans laquelle sont stockes toutes les images intermdiaires
   ncssaires  l'algorithme de corrlation. Actuellement, on alloue et
   rempli deux de ces structures (une 'droite' et une 'gauche', mais il serait
   peut-tre un peu mieux de n'en dterminer qu'une (cela permettrait de gagner
   de la place mmoire) */

typedef struct SP_IMAGES_TMP {
  SP_IMAGES_TMP_STATE	state;	/* tat de la structure */

  SP_IMAGES_SIZES	images_sizes; /* La structure ci-dessus, qui donne
					 toutes les tailles de images qui
					 suivent */

  /* Les images intermdiaires ncessaires aux filtrages */
  SP_IMAGE_STATE	state_intermediaire_haut;
  SP_IMAGE_STATE	state_intermediaire_bas;
  short*		intermediaire_haut;
  short*		intermediaire_bas;

  /* Les images ncessaires au calcul des moyennes et sigmas */
  SP_IMAGE_STATE	state_intermediaire_moyennes1;
  SP_IMAGE_STATE	state_intermediaire_moyennes2;
  short*		intermediaire_moyennes1;
  int*			intermediaire_moyennes2;

  /* Les images de moyennes et sigmas */
  SP_IMAGE_STATE	state_moyennes;
  SP_IMAGE_STATE	state_sigmas;
  float*		im_moyennes;
  float*		im_sigmas;


} SP_IMAGES_TMP;

/**********************************************************************/
/* Pour l'tat d'une structure 'SP_CORRELATION_PARAMS' */
typedef enum SP_CORRELATION_PARAMS_STATE {
  SP_CORRELATION_PARAMS_NOT_INITIALIZED,
  SP_CORRELATION_PARAMS_INITIALIZED} SP_CORRELATION_PARAMS_STATE;

/* Structure dans laquelle sont stocks les paramtres pour la fonction de
   corrlation par ZNCC */
typedef struct SP_CORRELATION_PARAMS{
  SP_CORRELATION_PARAMS_STATE	state;
  int		win_nb_lig;
  int		win_nb_col;	/* Taille de la fentre de corrlation */
  int		disp_min;
  int		disp_max;	/* cart de disparit */

  SP_TYPE_CORRELATION		type_correlation; /* Indique si on fait ZNCC (0)
						     ou CENSUS (2), et si on
						     dtermine les carts
						     types ou non (1) ou (3)*/
  SP_TYPE_INTERPOLATION_DISP	type_interpolation; /* Indique le type
						       d'interpolation */

  int		threads;	/* number of worker threads */
  void **	thread_array;	/* worker threads */

  /* 'th' stands for 'threshold'. Les diffrents seuils pour valider un
     apariement. Dans l'ordre : seuil sur l'cart type de la fentre considre,
     sur la valeur du score max, sur l'cart entre les 2 plus grand pics, sur la
     diffrence max / voisins, et sur l'cart (en pixels) corrlation directe /
     corrlation inverse. */
  double	var_th;
  double	score_th;
  double	diff_neighbor_max_th;
  int		inv_th;
} SP_CORRELATION_PARAMS;


/**********************************************************************/
/* Pour l'tat d'une structure 'SP_RECTIF_PARAMS' */
typedef enum SP_RECTIF_PARAMS_STATE {
  SP_RECTIF_PARAMS_NOT_INITIALIZED,
  SP_RECTIF_PARAMS_MATRICES_INITIALIZED} SP_RECTIF_PARAMS_STATE;

/* Structure dans laquelle sont stocks les matrices ncessaires  la
   rectification, et ventuellement les tables de rectification */
typedef struct SP_RECTIF_PARAMS{
  SP_RECTIF_PARAMS_STATE	state;

  /* Suivent toutes les matrices. Leur nom doit tre explicite, non ? */
  double		intrinseque[16];
  double		rectification[9];
  double		distortion[3];
  double		intrinseque_rectifiee[16];

  SP_IMAGES_SIZES	images_sizes;
  /* Cette structure (qui donne toutes les tailles de images) est utile pour
     la cration et l'initialisation des images d'incides ci-dessous. */

  short*		table_i;
  short*		table_j;
  short*		inverse_table_i;
  short*		inverse_table_j;
  /* Remarque : les images d'indices ont la taille des images aprs rduction
     (c'est au moment de la rectification que l'on considrera une ventuelle
     sous-image) */

} SP_RECTIF_PARAMS;


/**********************************************************************/
/* Pour l'tat d'une structure 'SP_CORRELATION_RESULTS' */
typedef enum SP_CORRELATION_RESULTS_STATE {
  SP_CORRELATION_RESULTS_NOT_ALLOCATED,
  SP_CORRELATION_RESULTS_ALLOCATED} SP_CORRELATION_RESULTS_STATE;

/* Structure  dans laquelle sont stockes tous les tableaux ncessaires  la
   fonction de corrlation ZNCC : elle comprend des tableaux intermdiaires et
   les tableaux 'rsultats' (ie. image de disparit et d'carts type sur la
   disparite) */

typedef struct SP_CORRELATION_RESULTS {
  SP_CORRELATION_RESULTS_STATE	state;	/* tat de la structure */
  int		win_nb_lig;	/* Nombre de lignes de la fentre de
				   corrlation */
  int		delta_max_disp;	/* cart de disparit maximum */
  int		disp_min_general; /* Disparit minimum */

  /* Suivent les diffrents tableaux ncessaires  la corrlation par ZNCC */
  int*		tab_disp_min;
  int*		tab_disp_max;
  int*		tab_disp_nj2_min;
  int*		tab_disp_nj2_max;

  void **	tab_tab_produits;

  void *	tab_scores;
  void *	tab_scores_2;
  int*		tab_max;

  void *	somme_pdt_bas;
  void *	somme_pdt_haut;

  SP_IMAGES_SIZES	images_sizes; /* La structure qui donne toutes les
					 tailles de images qui suivent */

  /* Toutes les images qui suivent ont la mme taille (nb_lig_final *
     nb_col_final) */

  /* L'image de disparits */
  SP_IMAGE_STATE	state_disparites;
  float*		im_disparites;

  /* L'image des carts types sur les disparits */
  SP_IMAGE_STATE	state_sigma_disparites;
  float*		im_sigma_disparites;

  /* L'image des rgions (remplie lors du blob filtrage) */
  SP_IMAGE_STATE	state_regions;
  int*			im_regions;

  /* The three varabiles follows indicate how well the correlation algorithm has
     performed */
  int nbRemovedByUnicity;	/* The number of matches for which the unicity
				   constraint has not been satisfied */
  int nbRemovedByBlob;		/* The number of matches that have been
				   discarded by the disparity blob filter */
  int nbCorrelated;		/* The number of pixels that have been
				   correlated */

} SP_CORRELATION_RESULTS;

/* Library error codes */
extern int	stereoPrintError(int num, char *errorCode, char *fileName,
			int lineNumber);

#define stereoError(num) (stereoPrintError(num, #num, __FILE__, __LINE__))
#define stereoPixel_id  511
#define stereoerr(m,n)	((((m) & 0x9FFF) << 16) | ((n) & 0xFFFF))

#define STEREO_OK 0

/* Les erreurs sur les tailles des images et fentres */
#define ERR_SP_TAUX_REDUCTION_NEGATIF		stereoerr(stereoPixel_id, 00)
#define ERR_SP_IMAGES_SIZES_NEGATIVES		stereoerr(stereoPixel_id, 01)
#define ERR_SP_IMAGES_SIZES_SOUS_IMAGE		stereoerr(stereoPixel_id, 02)
#define ERR_SP_IMAGES_SIZES_INVALIDES		stereoerr(stereoPixel_id, 03)
#define ERR_SP_IMAGES_SIZES_NOT_INITIALIZED	stereoerr(stereoPixel_id, 04)
#define ERR_SP_CORRELATION_RESULTS_NOT_ALLOCATED	stereoerr(stereoPixel_id, 05)
#define ERR_SP_CORRELATION_PARAMS_NOT_INITIALIZED	stereoerr(stereoPixel_id, 06)
#define ERR_SP_CORRELATION_RESULTS_NOT_COMPUTED		stereoerr(stereoPixel_id, 07)

/* Les erreurs sur les images et vecteurs passs en paramtres */
#define ERR_SP_IMAGE_SOURCE_INVALIDE		stereoerr(stereoPixel_id, 10)
#define ERR_SP_IMAGE_TMP_INVALIDE		stereoerr(stereoPixel_id, 11)
#define ERR_SP_CORRELATION_RESULTS_INVALIDE	stereoerr(stereoPixel_id, 12)
#define ERR_SP_VECTEUR_R8_INVALIDE		stereoerr(stereoPixel_id, 13)
#define ERR_SP_MATRICE_NULLE			stereoerr(stereoPixel_id, 14)
#define ERR_SP_RECTIF_PARAMS_STATE		stereoerr(stereoPixel_id, 15)
#define ERR_SP_IMAGE_DISPARITES_INVALIDE	stereoerr(stereoPixel_id, 16)
#define ERR_SP_IMAGE_TARGET_INVALIDE		stereoerr(stereoPixel_id, 17)

/* Les erreurs sur les paramtres des filtres */
#define ERR_SP_TAILLES_LAPLACIEN_NEGATIVES	stereoerr(stereoPixel_id, 20)
#define ERR_SP_TAILLES_LAPLACIEN_MAL_ORDONNEES	stereoerr(stereoPixel_id, 21)
#define ERR_SP_TAILLES_LAPLACIEN_PAIRES		stereoerr(stereoPixel_id, 22)
#define ERR_SP_COEFFICIENT_LAPLACIEN_NEGATIF	stereoerr(stereoPixel_id, 23)
#define ERR_SP_TYPE_CENSUS			stereoerr(stereoPixel_id, 25)

/* Les erreurs d'allocation */
#define ERR_SP_MALLOC_IMAGES_SIZES		stereoerr(stereoPixel_id, 30)
#define ERR_SP_MALLOC_IMAGES_TMP		stereoerr(stereoPixel_id, 31)
#define ERR_SP_MALLOC_CORRELATION_RESULTS	stereoerr(stereoPixel_id, 32)
#define ERR_SP_MALLOC_RECTIF_PARAMS		stereoerr(stereoPixel_id, 33)
#define ERR_SP_MALLOC_RECTIF_TABLE		stereoerr(stereoPixel_id, 34)
#define ERR_SP_MALLOC_RECTIF_INVERSE_TABLE	stereoerr(stereoPixel_id, 35)
#define ERR_SP_MALLOC_CORRELATION_PARAMS	stereoerr(stereoPixel_id, 36)
#define ERR_SP_CREATION_IMA1			stereoerr(stereoPixel_id, 37)
#define ERR_SP_CREATION_IMA2			stereoerr(stereoPixel_id, 38)
#define ERR_SP_CREATION_IMA4			stereoerr(stereoPixel_id, 39)
#define ERR_SP_ALLOC_IM3D			stereoerr(stereoPixel_id, 40)

/* Les erreurs sur les paramtres de corrlation */
#define ERR_SP_TAILLE_FENETRE_CORRELATION	stereoerr(stereoPixel_id, 50)
#define ERR_SP_ECART_DISPARITE			stereoerr(stereoPixel_id, 51)
#define ERR_SP_SEUIL_NEGATIF			stereoerr(stereoPixel_id, 52)
#define ERR_SP_TYPE_CORRELATION			stereoerr(stereoPixel_id, 53)

/* Les erreurs qui peuvent survenir en cours de traitement */
#define ERR_SP_TOO_MANY_BLOBS			stereoerr(stereoPixel_id, 60)

/* Les erreurs relatives aux fichiers */
#define ERR_SP_CREATE_FILE			stereoerr(stereoPixel_id, 70)

/* Les erreurs relatives aux fentre */
#define ERR_SP_NO_WINDOW			stereoerr(stereoPixel_id, 80)

/* Les erreurs diverses et varies */
#define ERR_SP_CAS_IMPOSSIBLE			stereoerr(stereoPixel_id, 90)
#define ERR_SP_FCT_NON_IMPLEMENTEE		stereoerr(stereoPixel_id, 91)


/* API */
SP_CORRELATION_PARAMS* copy_correlation_params ( SP_CORRELATION_PARAMS* correlation_params );
SP_CORRELATION_RESULTS* copy_correlation_results ( SP_CORRELATION_RESULTS* correlation_results );
SP_IMAGES_SIZES* copy_images_sizes ( SP_IMAGES_SIZES* images_sizes );
SP_IMAGES_TMP* copy_images_tmp ( SP_IMAGES_TMP* images_tmp );
SP_RECTIF_PARAMS* copy_rectif_params ( SP_RECTIF_PARAMS* rectif_params );
SP_CORRELATION_PARAMS* create_correlation_params ( void );
SP_CORRELATION_RESULTS* create_correlation_results ( void );
SP_IMAGES_SIZES* create_images_sizes ( void );
SP_IMAGES_TMP* create_images_tmp ( void );
SP_RECTIF_PARAMS* create_rectif_params ( void );
void destroy_correlation_params ( SP_CORRELATION_PARAMS* correlation_params );
void destroy_correlation_results ( SP_CORRELATION_RESULTS* correlation_results );
void destroy_images_sizes ( SP_IMAGES_SIZES* images_sizes );
void destroy_images_tmp ( SP_IMAGES_TMP* images_tmp );
void destroy_rectif_params ( SP_RECTIF_PARAMS* rectif_params );
int fill_inverse_tables ( SP_RECTIF_PARAMS* rectif_params );
int info_correlation_params ( SP_CORRELATION_PARAMS* correlation_params );
int info_correlation_results ( SP_CORRELATION_RESULTS* correlation_results );
int info_images_sizes ( SP_IMAGES_SIZES* images_sizes );
int info_images_tmp ( SP_IMAGES_TMP* images_tmp );
int info_rectif_params ( SP_RECTIF_PARAMS* rectif_params );

int	init_correlation_params(SP_CORRELATION_PARAMS* correlation_params,
		SP_TYPE_CORRELATION type_correlation,
		SP_TYPE_INTERPOLATION_DISP type_interpolation, int win_nb_lig,
		int win_nb_col, int disp_min, int disp_max, int threads,
		double var_th, double score_th, double diff_neighbor_max_th,
		int inv_th);

int init_correlation_results ( SP_IMAGES_SIZES* images_sizes, SP_CORRELATION_PARAMS* correlation_params, SP_CORRELATION_RESULTS* correlation_results );
int init_images_sizes ( SP_IMAGES_SIZES* images_sizes, int nblig, int nbcol, int previousReduction, int taux_reduction_i, int taux_reduction_j, double debut_ligne, double fin_ligne, double debut_colonne, double fin_colonne );
int init_images_tmp ( SP_IMAGES_TMP* images_tmp, SP_IMAGES_SIZES* images_sizes );
int init_rectif_params ( SP_RECTIF_PARAMS* rectif_params, SP_IMAGES_SIZES* images_sizes, double* mat_intrinseque, double* mat_rectification, double* mat_distortion, double* mat_intrinseque_rectifiee );
int init_rectif_params_inverse_tables ( SP_RECTIF_PARAMS* rectif_params );
int init_rectif_params_tables ( SP_CORRECTION_DISTORTION type_distortion, SP_RECTIF_PARAMS* rectif_params );
void sc_clean_correlation_results ( SP_CORRELATION_RESULTS *correlation_results );
void sc_clean_images_sizes ( SP_IMAGES_SIZES *images_sizes );
void sc_clean_images_tmp ( SP_IMAGES_TMP *images_tmp );

void calcule_disp_tableaux ( int nbcol, int nj2, int disparite_min, int disparite_max, int* tab_disp_max, int* tab_disp_min, int* tab_disp_nj2_max, int* tab_disp_nj2_min, int* disp_max_general, int* disp_min_general );

int	spix_correl_census(unsigned char *image_gauche,
		unsigned char *image_droite, SP_IMAGES_TMP *images_gauches,
		SP_IMAGES_TMP *images_droites,
		SP_CORRELATION_PARAMS *correlation_params,
		SP_CORRELATION_RESULTS *correlation_results);
int	spix_correl_zncc(unsigned char *source_gauche,
		unsigned char *source_droite, SP_IMAGES_TMP *images_gauches,
		SP_IMAGES_TMP *images_droites,
		SP_CORRELATION_PARAMS *correlation_params,
		SP_CORRELATION_RESULTS *correlation_results);

int moyennes_sigmas ( unsigned char* im_source, SP_IMAGES_TMP* images_tmp, SP_CORRELATION_PARAMS* correlation_params );

int blob_filter_disparites ( SP_CORRELATION_RESULTS* correlation_results, int sizeMin, float diffMax );
int old_reconstruction_3d ( SP_CORRELATION_RESULTS* correlation_results, SP_IMAGES_SIZES* images_size, double* matrice_reconstruction, double* matrice_extrinseque, double* matrice_intrinseque, DATA_IM3D* im3d );
int reconstruction_3d ( SP_CORRELATION_RESULTS* correlation_results, SP_IMAGES_SIZES* images_size, SP_REPERE_RECONSTRUCTION type_repere, SP_TYPE_RECONSTRUCTION type_recontruction, double* rect_2_cam, double* baseline, double* mat_intrinseque, double* mat_intrinseque_rectifiee, DATA_IM3D* im3d );

#ifdef __cplusplus
}
#endif
#endif /* H_CALIFE_STEREO_PIXEL */
